# Copyright (c) 2010-2011 Outpost Embedded, LLC
# This file is part of the pydbusclone software package.
#
# pydbusclone is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 2 as published by the
# Free Software Foundation.
#
# A copy of the license has been included in the COPYING file.

from threading import RLock

from dbus import DBusException


class DBusExceptionTranslator(object):
    _exception_types = None
    _lock = None

    def __init__(self):
        self._exception_types = {}
        self._lock = RLock()

    def get(self, name):
        if '.' not in name:
            raise ValueError('invalid dbus error name: %r' % name)
        with self._lock:
            try:
                return self._exception_types[name]
            except KeyError:
                exc_name = name.rsplit('.', 1)[-1]
                exc_type = type(exc_name, (DBusException,), {})
                self._exception_types[name] = exc_type
                return exc_type

    def convert(self, e):
        exc_type = self.get(e.get_dbus_name())
        e.__class__ = exc_type


translator = DBusExceptionTranslator()
get_dbus_exc = translator.get
convert_dbus_exc = translator.convert
