# Copyright (c) 2010-2011 Outpost Embedded, LLC
# This file is part of the pydbusclone software package.
#
# pydbusclone is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 2 as published by the
# Free Software Foundation.
#
# A copy of the license has been included in the COPYING file.

from PyQt4 import QtCore, QtGui


class FunctionObject(QtCore.QObject):
    instances = []

    trigger_timer_start = QtCore.pyqtSignal()

    __timer = None

    def __init__(self, fn, thread = None):
        super(FunctionObject, self).__init__()
        self.fn = fn

        # Prevent GC:
        type(self).instances.append(self)

        self.__timer = QtCore.QTimer()

        if (thread is not None) and (self.thread() != thread):
            self.moveToThread(thread)

    def moveToThread(self, thread):
        super(FunctionObject, self).moveToThread(thread)
        self.__timer.moveToThread(thread)

    def schedule(self, when = 0, single_shot = True):
        self.__timer.setInterval(when)
        self.__timer.setSingleShot(single_shot)
        self.__timer.timeout.connect(self.call_fn)
        if single_shot:
            self.__timer.timeout.connect(
              lambda: type(self).instances.remove(self)
            )
        self.trigger_timer_start.connect(self.__timer.start)
        self.trigger_timer_start.emit()

    @QtCore.pyqtSlot()
    def call_fn(self):
        self.fn()

    @classmethod
    def call_in_thread(cls, fn, thread):
        cls(fn, thread).schedule()

    @classmethod
    def call_in_main_thread(cls, fn):
        cls.call_in_thread(fn, QtGui.QApplication.instance().thread())


def call_in_main_thread(fn, *args, **kwargs):
    FunctionObject.call_in_main_thread(lambda: fn(*args, **kwargs))
