# Copyright (c) 2010-2011 Outpost Embedded, LLC
# This file is part of the pydbusclone software package.
#
# pydbusclone is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 2 as published by the
# Free Software Foundation.
#
# A copy of the license has been included in the COPYING file.

import Queue


class CallbackList(object):
    callbacks = None

    def __init__(self):
        self.callbacks = []

    def add(self, fn):
        if fn in self.callbacks:
            raise ValueError(fn)
        self.callbacks.append(fn)

    def remove(self, fn):
        self.callbacks.remove(fn)

    def call(self, *args, **kwargs):
        for fn in self.callbacks:
            fn(*args, **kwargs)


class MultiCallHandler(object):
    _numcalls = None
    _callback = None
    _pending_replies = None
    _replies = None
    _errors = None

    def __init__(self, numcalls, callback):
        if numcalls <= 0:
            raise ValueError(numcalls)

        self._numcalls = numcalls
        self._callback = callback
        self._pending_replies = numcalls
        self._replies = []
        self._errors = []

    def reply_handler(self, *args):
        self._decrement()
        self._replies.append(args)
        self._maybe_call_callback()

    def error_handler(self, e):
        self._decrement()
        self._errors.append(e)
        self._maybe_call_callback()

    def _decrement(self):
        self._pending_replies -= 1

    def _maybe_call_callback(self):
        if self._pending_replies == 0:
            self._callback(self._replies, self._errors)
        elif self._pending_replies < 0:
            raise AssertionError(self._pending_replies)


class MultiCallWaiter(MultiCallHandler):
    _queue = None

    def __init__(self, numcalls):
        self._queue = Queue.Queue()
        super(MultiCallWaiter, self).__init__(
          numcalls,
          (lambda *a: self._queue.put(a)),
        )

    def get_results(self):
        return self._queue.get()
